# Note: this file has been initially generated by Module::Build::Convert 0.49

use strict;
use warnings;

use Module::Build;

my $build = Module::Build->new
   (
    module_name => 'File::Flock',
    dist_abstract => 'Wrapper for flock() to make file locking trivial',
    dist_author => 'David Muir Sharnoff <cpan@dave.sharnoff.org>',
    dist_version_from => 'lib/File/Flock.pm',
    requires => {
        'AnyEvent' => 0,
        'Data::Structure::Util' => 0,
        'IO::Event' => '0.812',
        'Time::HiRes' => 0,
    },
    build_requires => {
        'File::Slurp' => 0,
        'Time::HiRes' => 0,
        'Test::SharedFork'  => 0,
    },
    meta_merge => {
        resources  => {
            repository => 'http://github.com/muir/File-Flock',
        },
    },
    license => 'unknown',
    create_readme => 1,
    create_makefile_pl => 'traditional',
   );

$build->create_build_script;

